# -*- coding: utf-8 -*-
from functools import partial
from pathlib import Path
import webbrowser

from PyQt5 import QtGui
from PyQt5 import QtWidgets
from PyQt5 import QtCore
import pyqtgraph as pg

from ...mdf import MDF, SUPPORTED_VERSIONS
from ..ui.main_window import Ui_PyMDFMainWindow
from ...version import __version__ as libversion
from ..utils import TERMINATED, run_thread_with_progress, setup_progress
from .list import ListWidget
from .file import FileWidget


class MainWindow(Ui_PyMDFMainWindow, QtWidgets.QMainWindow):

    def __init__(self, files=None, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.setupUi(self)
        self._settings = QtCore.QSettings()
        self._light_palette = self.palette()

        self.progress = None

        self.files.tabCloseRequested.connect(self.close_file)

        self.concatenate.toggled.connect(self.function_select)
        self.cs_btn.clicked.connect(self.cs_clicked)
        self.cs_format.insertItems(0, SUPPORTED_VERSIONS)
        self.cs_compression.insertItems(
            0, ("no compression", "deflate", "transposed deflate")
        )
        self.cs_split_size.setValue(10)

        self.files_list = ListWidget(self)
        self.files_list.setDragDropMode(QtWidgets.QAbstractItemView.InternalMove)
        self.files_layout.addWidget(self.files_list, 0, 0, 1, 2)
        self.files_list.itemDoubleClicked.connect(self.delete_item)

        menu = self.menubar.addMenu("File")
        open_group = QtWidgets.QActionGroup(self)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/open.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        action = QtWidgets.QAction(icon, "Open", menu)
        action.triggered.connect(self.open)
        open_group.addAction(action)
        menu.addActions(open_group.actions())

        # mode_actions
        mode_actions = QtWidgets.QActionGroup(self)

        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/file.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        action = QtWidgets.QAction(icon, "{: <20}".format("Single files"), menu)
        action.triggered.connect(partial(self.stackedWidget.setCurrentIndex, 0))
        mode_actions.addAction(action)

        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/list.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        action = QtWidgets.QAction(icon, "{: <20}".format("Multiple files"), menu)
        action.triggered.connect(partial(self.stackedWidget.setCurrentIndex, 1))
        mode_actions.addAction(action)

        menu = QtWidgets.QMenu("Mode", self.menubar)
        menu.addActions(mode_actions.actions())
        self.menubar.addMenu(menu)

        menu = QtWidgets.QMenu("Settings", self.menubar)
        self.menubar.addMenu(menu)

        # search mode menu
        search_option = QtWidgets.QActionGroup(self)

        for option in ("Match start", "Match contains"):

            action = QtWidgets.QAction(option, menu)
            action.setCheckable(True)
            search_option.addAction(action)
            action.triggered.connect(partial(self.set_search_option, option))

            if option == self._settings.value('search_match', "Match start"):
                action.setChecked(True)

        submenu = QtWidgets.QMenu("Search", self.menubar)
        submenu.addActions(search_option.actions())
        menu.addMenu(submenu)

        # search mode menu
        search_option = QtWidgets.QActionGroup(self)

        for option in ("Disabled", "Enabled"):

            action = QtWidgets.QAction(option, menu)
            action.setCheckable(True)
            search_option.addAction(action)
            action.triggered.connect(partial(self.set_subplot_option, option))

            if option == self._settings.value('subplots', "Disabled"):
                action.setChecked(True)

        submenu = QtWidgets.QMenu("Sub-plots", self.menubar)
        submenu.addActions(search_option.actions())
        menu.addMenu(submenu)


        # search mode menu
        search_option = QtWidgets.QActionGroup(self)

        for option in ("Disabled", "Enabled"):

            action = QtWidgets.QAction(option, menu)
            action.setCheckable(True)
            search_option.addAction(action)
            action.triggered.connect(partial(self.set_subplot_link_option, option))

            if option == self._settings.value('subplots_link', "Disabled"):
                action.setChecked(True)

        submenu = QtWidgets.QMenu("Link sub-plots X-axis", self.menubar)
        submenu.addActions(search_option.actions())
        menu.addMenu(submenu)

        # search mode menu
        plot_background_option = QtWidgets.QActionGroup(self)

        for option in ("Black", "White"):

            action = QtWidgets.QAction(option, menu)
            action.setCheckable(True)
            plot_background_option.addAction(action)
            action.triggered.connect(partial(self.set_plot_background, option))

            if option == self._settings.value('plot_background', "Black"):
                action.setChecked(True)
                action.triggered.emit()

        submenu = QtWidgets.QMenu("Plot background", self.menubar)
        submenu.addActions(plot_background_option.actions())
        menu.addMenu(submenu)

        # search mode menu
        theme_option = QtWidgets.QActionGroup(self)

        for option in ("Dark", "Light"):

            action = QtWidgets.QAction(option, menu)
            action.setCheckable(True)
            theme_option.addAction(action)
            action.triggered.connect(partial(self.set_theme, option))

            if option == self._settings.value('theme', "Light"):
                action.setChecked(True)
                action.triggered.emit()

        submenu = QtWidgets.QMenu("Theme", self.menubar)
        submenu.addActions(theme_option.actions())
        menu.addMenu(submenu)

        # plot option menu
        plot_actions = QtWidgets.QActionGroup(self)

        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/fit.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        action = QtWidgets.QAction(icon, f"{'Fit trace': <20}\tF", menu)
        action.triggered.connect(partial(self.plot_action, key=QtCore.Qt.Key_F))
        action.setShortcut(QtCore.Qt.Key_F)
        plot_actions.addAction(action)

        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/grid.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        action = QtWidgets.QAction(icon, "{: <20}\tG".format("Grid"), menu)
        action.triggered.connect(partial(self.plot_action, key=QtCore.Qt.Key_G))
        action.setShortcut(QtCore.Qt.Key_G)
        plot_actions.addAction(action)

        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/home.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        action = QtWidgets.QAction(icon, "{: <20}\tH".format("Home"), menu)
        action.triggered.connect(partial(self.plot_action, key=QtCore.Qt.Key_H))
        action.setShortcut(QtCore.Qt.Key_H)
        plot_actions.addAction(action)

        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/list2.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        action = QtWidgets.QAction(icon, "{: <20}\tS".format("Stack"), menu)
        action.triggered.connect(partial(self.plot_action, key=QtCore.Qt.Key_S))
        action.setShortcut(QtCore.Qt.Key_S)
        plot_actions.addAction(action)

        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/zoom-in.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        action = QtWidgets.QAction(icon, "{: <20}\tI".format("Zoom in"), menu)
        action.triggered.connect(partial(self.plot_action, key=QtCore.Qt.Key_I))
        action.setShortcut(QtCore.Qt.Key_I)
        plot_actions.addAction(action)

        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/zoom-out.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        action = QtWidgets.QAction(icon, "{: <20}\tO".format("Zoom out"), menu)
        action.triggered.connect(partial(self.plot_action, key=QtCore.Qt.Key_O))
        action.setShortcut(QtCore.Qt.Key_O)
        plot_actions.addAction(action)

        action = QtWidgets.QAction("{: <20}\t.".format("Toggle dots"), menu)
        action.triggered.connect(partial(self.toggle_dots, key=QtCore.Qt.Key_O))
        action.setShortcut(QtCore.Qt.Key_Period)
        plot_actions.addAction(action)

        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/plus.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        action = QtWidgets.QAction(icon, "{: <20}\tIns".format("Insert computation"), menu)
        action.triggered.connect(partial(self.plot_action, key=QtCore.Qt.Key_Insert))
        action.setShortcut(QtCore.Qt.Key_Insert)
        plot_actions.addAction(action)

        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/save.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        action = QtWidgets.QAction(icon, "{: <20}\tCtrl+S".format("Save active subplot channels"), menu)
        action.triggered.connect(partial(self.plot_action, key=QtCore.Qt.Key_S, modifier=QtCore.Qt.ControlModifier))
        action.setShortcut(QtGui.QKeySequence("Ctrl+S"))
        plot_actions.addAction(action)

        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/save.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        action = QtWidgets.QAction(icon, "{: <20}\tCtrl+Shift+S".format("Save all subplot channels"), menu)
        action.triggered.connect(self.save_all_subplots)
        action.setShortcut(QtGui.QKeySequence("Ctrl+Shift+S"))
        plot_actions.addAction(action)


        # values display

        display_format_actions = QtWidgets.QActionGroup(self)

        action = QtWidgets.QAction("{: <20}\tCtrl+H".format("Hex"), menu)
        action.triggered.connect(
            partial(self.plot_action, key=QtCore.Qt.Key_H, modifier=QtCore.Qt.ControlModifier)
        )
        action.setShortcut(QtGui.QKeySequence("Ctrl+H"))
        display_format_actions.addAction(action)

        action = QtWidgets.QAction("{: <20}\tCtrl+B".format("Bin"), menu)
        action.triggered.connect(
            partial(self.plot_action, key=QtCore.Qt.Key_B, modifier=QtCore.Qt.ControlModifier)
        )
        action.setShortcut(QtGui.QKeySequence("Ctrl+B"))
        display_format_actions.addAction(action)

        action = QtWidgets.QAction("{: <20}\tCtrl+P".format("Physical"), menu)
        action.triggered.connect(
            partial(self.plot_action, key=QtCore.Qt.Key_P, modifier=QtCore.Qt.ControlModifier)
        )
        action.setShortcut(QtGui.QKeySequence("Ctrl+P"))
        display_format_actions.addAction(action)

        # info

        info = QtWidgets.QActionGroup(self)

        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/info.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        action = QtWidgets.QAction(icon, "{: <20}\tM".format("Statistics"), menu)
        action.triggered.connect(partial(self.plot_action, key=QtCore.Qt.Key_M))
        action.setShortcut(QtGui.QKeySequence("M"))
        info.addAction(action)

        # sub_plots

        subs = QtWidgets.QActionGroup(self)

        action = QtWidgets.QAction("{: <20}\tShift+C".format("Cascade sub-plots"), menu)
        action.triggered.connect(partial(self.show_sub_windows, mode='cascade'))
        action.setShortcut(QtGui.QKeySequence("Shift+C"))
        subs.addAction(action)

        action = QtWidgets.QAction("{: <20}\tShift+T".format("Tile sub-plots"), menu)
        action.triggered.connect(partial(self.show_sub_windows, mode='tile'))
        action.setShortcut(QtGui.QKeySequence("Shift+T"))
        subs.addAction(action)

        # cursors
        cursors_actions = QtWidgets.QActionGroup(self)

        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/cursor.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        action = QtWidgets.QAction(icon, "{: <20}\tC".format("Cursor"), menu)
        action.triggered.connect(partial(self.plot_action, key=QtCore.Qt.Key_C))
        action.setShortcut(QtCore.Qt.Key_C)
        cursors_actions.addAction(action)

        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/right.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        action = QtWidgets.QAction(icon, "{: <20}\t←".format("Move cursor left"), menu)
        action.triggered.connect(partial(self.plot_action, key=QtCore.Qt.Key_Left))
        action.setShortcut(QtCore.Qt.Key_Left)
        cursors_actions.addAction(action)

        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/left.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        action = QtWidgets.QAction(icon, "{: <20}\t→".format("Move cursor right"), menu)
        action.triggered.connect(partial(self.plot_action, key=QtCore.Qt.Key_Right))
        action.setShortcut(QtCore.Qt.Key_Right)
        cursors_actions.addAction(action)

        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/range.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        action = QtWidgets.QAction(icon, "{: <20}\tR".format("Range"), menu)
        action.triggered.connect(partial(self.plot_action, key=QtCore.Qt.Key_R))
        action.setShortcut(QtCore.Qt.Key_R)
        cursors_actions.addAction(action)

        menu = QtWidgets.QMenu("Plot", self.menubar)
        menu.addActions(plot_actions.actions())
        menu.addSeparator()
        menu.addActions(cursors_actions.actions())
        menu.addSeparator()
        menu.addActions(display_format_actions.actions())
        menu.addSeparator()
        menu.addActions(subs.actions())
        menu.addSeparator()
        menu.addActions(info.actions())
        self.menubar.addMenu(menu)

        menu = self.menubar.addMenu("Help")
        open_group = QtWidgets.QActionGroup(self)
        action = QtWidgets.QAction("Online documentation", menu)
        action.triggered.connect(self.help)
        open_group.addAction(action)
        menu.addActions(open_group.actions())

        self.with_dots = self._settings.value('dots', False, type=bool)
        self.stackedWidget.setCurrentIndex(0)
        self.setWindowTitle(f'asammdf {libversion}')

        if files:
            for name in files:
                self._open_file(name)

        self.setAcceptDrops(True)

        self.show()

        self.set_search_option(self._settings.value('search_match', "Match start"))
        self.set_subplot_option(self._settings.value('subplots', "Disabled"))
        self.set_subplot_link_option(self._settings.value('subplots_link', "Disabled"))

    def help(self, event):
        webbrowser.open_new(r'http://asammdf.readthedocs.io/en/development/gui.html')

    def save_all_subplots(self, key):
        widget = self.files.currentWidget()
        widget.save_all_subplots()

    def plot_action(self, key, modifier=QtCore.Qt.NoModifier):
        event = QtGui.QKeyEvent(QtCore.QEvent.KeyPress, key, modifier)
        widget = self.files.currentWidget()
        if widget and widget.get_current_plot():
            widget.get_current_plot().keyPressEvent(event)

    def toggle_dots(self, key):
        self.with_dots = not self.with_dots
        self._settings.setValue('dots', self.with_dots)

        count = self.files.count()

        for i in range(count):
            self.files.widget(i).set_line_style(with_dots=self.with_dots)

    def show_sub_windows(self, mode):

        widget = self.files.currentWidget()
        if widget:
            if mode == 'tile':
                widget.mdi_area.tileSubWindows()
            elif mode == 'cascade':
                widget.mdi_area.cascadeSubWindows()

    def set_search_option(self, option):
        self.match = option
        self._settings.setValue('search_match', self.match)
        count = self.files.count()
        for i in range(count):
            self.files.widget(i).search_field.set_search_option(option)
            self.files.widget(i).filter_field.set_search_option(option)

    def set_subplot_option(self, option):
        self.subplots = option == 'Enabled'
        self._settings.setValue('subplots', option)

    def set_plot_background(self, option):
        self._settings.setValue('plot_background', option)
        if option == 'Black':
            pg.setConfigOption('background', 'k')
            pg.setConfigOption('foreground', 'w')
        else:
            pg.setConfigOption('background', 'w')
            pg.setConfigOption('foreground', 'k')

    def set_theme(self, option):
        self._settings.setValue('theme', option)
        app = QtWidgets.QApplication.instance()
        if option == 'Light':
            app.setPalette(self._light_palette)
        else:

            palette = QtGui.QPalette()
            brush = QtGui.QBrush(QtGui.QColor(255, 255, 255))
            brush.setStyle(QtCore.Qt.SolidPattern)
            palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.WindowText, brush)
            brush = QtGui.QBrush(QtGui.QColor(55, 55, 55))
            brush.setStyle(QtCore.Qt.SolidPattern)
            palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Button, brush)
            brush = QtGui.QBrush(QtGui.QColor(82, 82, 82))
            brush.setStyle(QtCore.Qt.SolidPattern)
            palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Light, brush)
            brush = QtGui.QBrush(QtGui.QColor(68, 68, 68))
            brush.setStyle(QtCore.Qt.SolidPattern)
            palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Midlight, brush)
            brush = QtGui.QBrush(QtGui.QColor(27, 27, 27))
            brush.setStyle(QtCore.Qt.SolidPattern)
            palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Dark, brush)
            brush = QtGui.QBrush(QtGui.QColor(36, 36, 36))
            brush.setStyle(QtCore.Qt.SolidPattern)
            palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Mid, brush)
            brush = QtGui.QBrush(QtGui.QColor(255, 255, 255))
            brush.setStyle(QtCore.Qt.SolidPattern)
            palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Text, brush)
            brush = QtGui.QBrush(QtGui.QColor(255, 255, 255))
            brush.setStyle(QtCore.Qt.SolidPattern)
            palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.BrightText, brush)
            brush = QtGui.QBrush(QtGui.QColor(255, 255, 255))
            brush.setStyle(QtCore.Qt.SolidPattern)
            palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.ButtonText, brush)
            brush = QtGui.QBrush(QtGui.QColor(0, 0, 0))
            brush.setStyle(QtCore.Qt.SolidPattern)
            palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Base, brush)
            brush = QtGui.QBrush(QtGui.QColor(55, 55, 55))
            brush.setStyle(QtCore.Qt.SolidPattern)
            palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Window, brush)
            brush = QtGui.QBrush(QtGui.QColor(0, 0, 0))
            brush.setStyle(QtCore.Qt.SolidPattern)
            palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Shadow, brush)
            brush = QtGui.QBrush(QtGui.QColor(27, 27, 27))
            brush.setStyle(QtCore.Qt.SolidPattern)
            palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.AlternateBase, brush)
            brush = QtGui.QBrush(QtGui.QColor(255, 255, 220))
            brush.setStyle(QtCore.Qt.SolidPattern)
            palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.ToolTipBase, brush)
            brush = QtGui.QBrush(QtGui.QColor(0, 0, 0))
            brush.setStyle(QtCore.Qt.SolidPattern)
            palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.ToolTipText, brush)
            brush = QtGui.QBrush(QtGui.QColor(255, 255, 255))
            brush.setStyle(QtCore.Qt.SolidPattern)
            palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.WindowText, brush)
            brush = QtGui.QBrush(QtGui.QColor(55, 55, 55))
            brush.setStyle(QtCore.Qt.SolidPattern)
            palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Button, brush)
            brush = QtGui.QBrush(QtGui.QColor(82, 82, 82))
            brush.setStyle(QtCore.Qt.SolidPattern)
            palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Light, brush)
            brush = QtGui.QBrush(QtGui.QColor(68, 68, 68))
            brush.setStyle(QtCore.Qt.SolidPattern)
            palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Midlight, brush)
            brush = QtGui.QBrush(QtGui.QColor(27, 27, 27))
            brush.setStyle(QtCore.Qt.SolidPattern)
            palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Dark, brush)
            brush = QtGui.QBrush(QtGui.QColor(36, 36, 36))
            brush.setStyle(QtCore.Qt.SolidPattern)
            palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Mid, brush)
            brush = QtGui.QBrush(QtGui.QColor(255, 255, 255))
            brush.setStyle(QtCore.Qt.SolidPattern)
            palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Text, brush)
            brush = QtGui.QBrush(QtGui.QColor(255, 255, 255))
            brush.setStyle(QtCore.Qt.SolidPattern)
            palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.BrightText, brush)
            brush = QtGui.QBrush(QtGui.QColor(255, 255, 255))
            brush.setStyle(QtCore.Qt.SolidPattern)
            palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.ButtonText, brush)
            brush = QtGui.QBrush(QtGui.QColor(0, 0, 0))
            brush.setStyle(QtCore.Qt.SolidPattern)
            palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Base, brush)
            brush = QtGui.QBrush(QtGui.QColor(55, 55, 55))
            brush.setStyle(QtCore.Qt.SolidPattern)
            palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Window, brush)
            brush = QtGui.QBrush(QtGui.QColor(0, 0, 0))
            brush.setStyle(QtCore.Qt.SolidPattern)
            palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Shadow, brush)
            brush = QtGui.QBrush(QtGui.QColor(27, 27, 27))
            brush.setStyle(QtCore.Qt.SolidPattern)
            palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.AlternateBase, brush)
            brush = QtGui.QBrush(QtGui.QColor(255, 255, 220))
            brush.setStyle(QtCore.Qt.SolidPattern)
            palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.ToolTipBase, brush)
            brush = QtGui.QBrush(QtGui.QColor(0, 0, 0))
            brush.setStyle(QtCore.Qt.SolidPattern)
            palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.ToolTipText, brush)
            brush = QtGui.QBrush(QtGui.QColor(27, 27, 27))
            brush.setStyle(QtCore.Qt.SolidPattern)
            palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.WindowText, brush)
            brush = QtGui.QBrush(QtGui.QColor(55, 55, 55))
            brush.setStyle(QtCore.Qt.SolidPattern)
            palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Button, brush)
            brush = QtGui.QBrush(QtGui.QColor(82, 82, 82))
            brush.setStyle(QtCore.Qt.SolidPattern)
            palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Light, brush)
            brush = QtGui.QBrush(QtGui.QColor(68, 68, 68))
            brush.setStyle(QtCore.Qt.SolidPattern)
            palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Midlight, brush)
            brush = QtGui.QBrush(QtGui.QColor(27, 27, 27))
            brush.setStyle(QtCore.Qt.SolidPattern)
            palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Dark, brush)
            brush = QtGui.QBrush(QtGui.QColor(36, 36, 36))
            brush.setStyle(QtCore.Qt.SolidPattern)
            palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Mid, brush)
            brush = QtGui.QBrush(QtGui.QColor(27, 27, 27))
            brush.setStyle(QtCore.Qt.SolidPattern)
            palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Text, brush)
            brush = QtGui.QBrush(QtGui.QColor(255, 255, 255))
            brush.setStyle(QtCore.Qt.SolidPattern)
            palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.BrightText, brush)
            brush = QtGui.QBrush(QtGui.QColor(27, 27, 27))
            brush.setStyle(QtCore.Qt.SolidPattern)
            palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.ButtonText, brush)
            brush = QtGui.QBrush(QtGui.QColor(55, 55, 55))
            brush.setStyle(QtCore.Qt.SolidPattern)
            palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Base, brush)
            brush = QtGui.QBrush(QtGui.QColor(55, 55, 55))
            brush.setStyle(QtCore.Qt.SolidPattern)
            palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Window, brush)
            brush = QtGui.QBrush(QtGui.QColor(0, 0, 0))
            brush.setStyle(QtCore.Qt.SolidPattern)
            palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Shadow, brush)
            brush = QtGui.QBrush(QtGui.QColor(55, 55, 55))
            brush.setStyle(QtCore.Qt.SolidPattern)
            palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.AlternateBase, brush)
            brush = QtGui.QBrush(QtGui.QColor(255, 255, 220))
            brush.setStyle(QtCore.Qt.SolidPattern)
            palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.ToolTipBase, brush)
            brush = QtGui.QBrush(QtGui.QColor(0, 0, 0))
            brush.setStyle(QtCore.Qt.SolidPattern)
            palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.ToolTipText, brush)
            app.setPalette(palette)

    def set_subplot_link_option(self, option):
        self.subplots_link = option == 'Enabled'
        self._settings.setValue('subplots_link', option)
        count = self.files.count()

        for i in range(count):
            self.files.widget(i).set_subplots_link(self.subplots_link)

    def update_progress(self, current_index, max_index):
        self.progress = current_index, max_index

    def delete_item(self, item):
        index = self.files_list.row(item)
        self.files_list.takeItem(index)

    def function_select(self, val):
        if self.concatenate.isChecked():
            self.cs_btn.setText("Concatenate")
        else:
            self.cs_btn.setText("Stack")

    def cs_clicked(self, event):
        if self.concatenate.isChecked():
            func = MDF.concatenate
            operation = "Concatenating"
        else:
            func = MDF.stack
            operation = "Stacking"

        version = self.cs_format.currentText()

        sync = self.sync.checkState() == QtCore.Qt.Checked
        add_samples_origin = self.add_samples_origin.checkState() == QtCore.Qt.Checked

        if version < "4.00":
            filter = "MDF version 3 files (*.dat *.mdf)"
        else:
            filter = "MDF version 4 files (*.mf4)"

        split = self.cs_split.checkState() == QtCore.Qt.Checked
        if split:
            split_size = int(self.cs_split_size.value() * 1024 * 1024)
        else:
            split_size = 0

        compression = self.cs_compression.currentIndex()

        count = self.files_list.count()

        files = [self.files_list.item(row).text() for row in range(count)]

        file_name, _ = QtWidgets.QFileDialog.getSaveFileName(
            self, "Select output measurement file", "", filter
        )

        if file_name:

            progress = setup_progress(
                parent=self,
                title=f"{operation} measurements",
                message=f"{operation} files and saving to {version} format",
                icon_name="stack",
            )

            target = func
            kwargs = {
                "files": files,
                "outversion": version,
                "callback": self.update_progress,
                "sync": sync,
                "add_samples_origin": add_samples_origin,
            }

            mdf = run_thread_with_progress(
                self,
                target=target,
                kwargs=kwargs,
                factor=50,
                offset=0,
                progress=progress,
            )

            if mdf is TERMINATED:
                progress.cancel()
                return

            mdf.configure(write_fragment_size=split_size)

            # save it
            progress.setLabelText(f'Saving output file "{file_name}"')

            target = mdf.save
            kwargs = {"dst": file_name, "compression": compression, "overwrite": True}

            run_thread_with_progress(
                self,
                target=target,
                kwargs=kwargs,
                factor=50,
                offset=50,
                progress=progress,
            )

            progress.cancel()

    def open_multiple_files(self, event):
        file_names, _ = QtWidgets.QFileDialog.getOpenFileNames(
            self,
            "Select measurement file",
            "",
            "MDF v3 (*.dat *.mdf);;MDF v4(*.mf4);;DL3/ERG files (*.dl3 *.erg);;All files (*.dat *.mdf *.mf4 *.dl3 *.erg)",
            "All files (*.dat *.mdf *.mf4 *.dl3 *.erg)",
        )

        if file_names:
            self.files_list.addItems(file_names)
            count = self.files_list.count()

            icon = QtGui.QIcon()
            icon.addPixmap(QtGui.QPixmap(":/file.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)

            for row in range(count):
                self.files_list.item(row).setIcon(icon)

    def open(self, event):
        if self.stackedWidget.currentIndex() == 0:
            self.open_file(event)
        else:
            self.open_multiple_files(event)

    def _open_file(self, file_name):
        file_name = Path(file_name)
        index = self.files.count()

        try:
            widget = FileWidget(file_name, self.with_dots, self.subplots, self.subplots_link, self)
            widget.search_field.set_search_option(self.match)
            widget.filter_field.set_search_option(self.match)
        except:
            raise
        else:
            self.files.addTab(widget, file_name.name)
            self.files.setTabToolTip(index, str(file_name))
            self.files.setCurrentIndex(index)
            widget.open_new_file.connect(self._open_file)

    def open_file(self, event):
        file_names, _ = QtWidgets.QFileDialog.getOpenFileNames(
            self,
            "Select measurement file",
            "",
            "MDF v3 (*.dat *.mdf);;MDF v4(*.mf4);;DL3/ERG files (*.dl3 *.erg);;All files (*.dat *.mdf *.mf4 *.dl3 *.erg)",
            "All files (*.dat *.mdf *.mf4 *.dl3 *.erg)",
        )

        for file_name in file_names:
            self._open_file(file_name)

    def close_file(self, index):
        widget = self.files.widget(index)
        if widget:
            widget.close()
            widget.setParent(None)

        if self.files.count():
            self.files.setCurrentIndex(0)

    def closeEvent(self, event):
        count = self.files.count()
        for i in range(count):
            self.files.widget(i).close()
        event.accept()

    def dragEnterEvent(self, e):

        e.accept()

    def dropEvent(self, e):
        try:
            for path in e.mimeData().text().splitlines():
                path = Path(path.replace(r'file:///', ''))
                if path.suffix.lower() in ('.dat', '.mdf', '.mf4'):
                    self._open_file(path)
        except:
            pass



# -*- coding: utf-8 -*-

from PyQt5 import QtGui
from PyQt5 import QtWidgets
from PyQt5 import QtCore

from ..ui import resource_rc as resource_rc
from ..ui.channel_display_widget import Ui_ChannelDiplay
from ..dialogs.range_editor import RangeEditor


class ChannelDisplay(Ui_ChannelDiplay, QtWidgets.QWidget):

    color_changed = QtCore.pyqtSignal(int, str)
    enable_changed = QtCore.pyqtSignal(int, int)
    ylink_changed = QtCore.pyqtSignal(int, int)

    __slots__ = (
        'color',
        '_value_prefix',
        '_value',
        '_name',
        'fmt',
        'index',
        'ranges',
        'unit',
        '_transparent',
    )

    def __init__(self, index, unit="", kind='f', precision=3, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.setupUi(self)

        self.color = "#ff0000"
        self._value_prefix = ""
        self._value = ""
        self._name = ""

        self.index = index
        self.ranges = {}
        self.unit = unit.strip()
        self.kind = kind
        self.precision = precision

        self._transparent = True

        self.color_btn.clicked.connect(self.select_color)
        self.display.stateChanged.connect(self.display_changed)
        self.ylink.stateChanged.connect(self.ylink_change)

        self.fm = QtGui.QFontMetrics(self.name.font())

        self.setToolTip(self._name)

        if kind in 'SUVui':
            self.fmt = '{}'
        else:
            self.fmt = f'{{:.{self.precision}f}}'

    def set_precision(self, precision):
        if self.kind == 'f':
            self.precision = precision
            self.fmt = f'{{:.{self.precision}f}}'

    def display_changed(self, state):
        state = self.display.checkState()
        self.enable_changed.emit(self.index, state)

    def ylink_change(self, state):
        state = self.ylink.checkState()
        self.ylink_changed.emit(self.index, state)

    def mouseDoubleClickEvent(self, event):
        dlg = RangeEditor(self.unit, self.ranges)
        dlg.exec_()
        if dlg.pressed_button == "apply":
            self.ranges = dlg.result

    def select_color(self):
        color = QtWidgets.QColorDialog.getColor(QtGui.QColor(self.color))
        if color.isValid():
            self.setColor(color.name())

            self.color_changed.emit(self.index, color.name())

    def setFmt(self, fmt):
        if self.kind in 'fSUV':
            pass
        else:
            if fmt == "hex":
                self.fmt = "0x{:X}"
            elif fmt == "bin":
                self.fmt = "0b{:b}"
            elif fmt == "phys":
                self.fmt = "{}"

    def setColor(self, color):
        self.color = color
        self.setName(self._name)
        self.setValue(self._value)
        self.color_btn.setStyleSheet(f"background-color: {color};")

    def setName(self, text=""):
        self.setToolTip(text)
        self._name = text

    def setPrefix(self, text=""):
        self._value_prefix = text

    def setValue(self, value):
        self._value = value
        if self.ranges and value not in ("", "n.a."):
            for (start, stop), color in self.ranges.items():
                if start <= value < stop:
                    self.setStyleSheet(f"background-color: {color};")
                    break
            else:
                self.setStyleSheet("background-color: transparent;")
        elif not self._transparent:
            self.setStyleSheet("background-color: transparent;")
        template = '{{}}{}'
        if value not in ("", "n.a."):
            template = template.format(self.fmt)
        else:
            template = template.format("{}")
        try:
            self.value.setText(template.format(self._value_prefix, value))
        except (ValueError, TypeError):
            template = '{}{}'
            self.value.setText(template.format(self._value_prefix, value))

    def keyPressEvent(self, event):
        key = event.key()
        modifier = event.modifiers()
        if modifier == QtCore.Qt.ControlModifier and key == QtCore.Qt.Key_C:
            QtWidgets.QApplication.instance().clipboard().setText(self._name)

        else:
            super().keyPressEvent(event)

    def resizeEvent(self, event):
        width = self.name.size().width()
        if self.unit:
            self.name.setText(self.fm.elidedText(f'{self._name} ({self.unit})', QtCore.Qt.ElideMiddle, width))
        else:
            self.name.setText(self.fm.elidedText(self._name, QtCore.Qt.ElideMiddle, width))

    def text(self):
        return self._name


# -*- coding: utf-8 -*-
import os

bin_ = bin
import logging
from functools import reduce, partial
import math

import numpy as np
from pathlib import Path

HERE = Path(__file__).resolve().parent

from ..ui import resource_rc as resource_rc
from .list import ListWidget

try:
    import pyqtgraph as pg

    from PyQt5 import QtGui
    from PyQt5 import QtWidgets
    from PyQt5 import QtCore

    from ..utils import COLORS
    from .cursor import Cursor
    from .formated_axis import FormatedAxis
    from ..dialogs.define_channel import DefineChannel
    from ...mdf import MDF
    from .list import ListWidget
    from .channel_display import ChannelDisplay
    from .channel_stats import ChannelStats

    if not hasattr(pg.InfiniteLine, "addMarker"):
        logger = logging.getLogger("asammdf")
        message = (
            "Old pyqtgraph package: Please install the latest pyqtgraph from the "
            "github develop branch\n"
            "pip install -I --no-deps "
            "https://github.com/pyqtgraph/pyqtgraph/archive/develop.zip"
        )
        logger.warning(message)

    class Plot(QtWidgets.QWidget):

        add_channels_request = QtCore.pyqtSignal(list)
        close_request = QtCore.pyqtSignal()
        clicked = QtCore.pyqtSignal()

        def __init__(self, signals, with_dots=False, *args, **kwargs):
            super().__init__(*args, **kwargs)
            self.setContentsMargins(0, 0, 0, 0)

            self.info_index = None

            main_layout = QtWidgets.QVBoxLayout(self)
            # self.setLayout(main_layout)

            vbox = QtWidgets.QVBoxLayout()
            widget = QtWidgets.QWidget()
            self.channel_selection = ListWidget()
            self.channel_selection.setAlternatingRowColors(False)
            self.channel_selection.setDragDropMode(QtWidgets.QAbstractItemView.InternalMove)
            hbox = QtWidgets.QHBoxLayout()
            hbox.addWidget(QtWidgets.QLabel("Cursor/Range information"))
            self.cursor_info = QtWidgets.QLabel("")
            self.cursor_info.setTextFormat(QtCore.Qt.RichText)
            self.cursor_info.setAlignment(
                QtCore.Qt.AlignRight | QtCore.Qt.AlignTrailing | QtCore.Qt.AlignVCenter
            )
            hbox.addWidget(self.cursor_info)
            vbox.addLayout(hbox)
            vbox.addWidget(self.channel_selection)
            widget.setLayout(vbox)

            self.splitter = QtWidgets.QSplitter()
            self.splitter.addWidget(widget)

            self.plot = _Plot(with_dots=with_dots, parent=self)
            self.plot.range_modified.connect(self.range_modified)
            self.plot.range_removed.connect(self.range_removed)
            self.plot.range_modified_finished.connect(self.range_modified_finished)
            self.plot.cursor_removed.connect(self.cursor_removed)
            self.plot.cursor_moved.connect(self.cursor_moved)
            self.plot.cursor_move_finished.connect(self.cursor_move_finished)
            self.plot.xrange_changed.connect(self.xrange_changed)
            self.plot.computation_channel_inserted.connect(self.computation_channel_inserted)
            self.plot.curve_clicked.connect(self.channel_selection.setCurrentRow)

            self.splitter.addWidget(self.plot)

            self._available_index = 0

            self.info = ChannelStats()
            self.splitter.addWidget(self.info)

            self.channel_selection.itemsDeleted.connect(self.channel_selection_reduced)
            self.channel_selection.itemPressed.connect(
                self.channel_selection_modified
            )
            self.channel_selection.currentRowChanged.connect(
                self.channel_selection_row_changed
            )
            self.channel_selection.add_channels_request.connect(self.add_channels_request)

            main_layout.addWidget(self.splitter)

            if signals:
                self.add_new_channels(signals)
                self.plot.set_current_index(self.plot.signals[0]._index)

            self.info.hide()

            self.splitter.setStretchFactor(0, 1)
            self.splitter.setStretchFactor(1, 2)
            self.splitter.setStretchFactor(2, 0)

        def mousePressEvent(self, event):
            self.clicked.emit()
            super().mousePressEvent(event)

        def channel_selection_modified(self, item):
            if item:
                index = self.channel_selection.itemWidget(item).index
                self.info_index = index

                sig, _ = self.plot.signal_by_index(index)
                if sig.enable:

                    self.plot.set_current_index(self.info_index)
                    stats = self.plot.get_stats(self.info_index)
                    self.info.set_stats(stats)

        def channel_selection_row_changed(self, row):
            if row >= 0:
                item = self.channel_selection.item(row)
                index = self.channel_selection.itemWidget(item).index
                self.info_index = index

                sig, _ = self.plot.signal_by_index(index)
                if sig.enable:

                    self.plot.set_current_index(self.info_index)
                    stats = self.plot.get_stats(self.info_index)
                    self.info.set_stats(stats)

        def channel_selection_reduced(self, deleted):
            for i in sorted(deleted, reverse=True):
                item = self.plot.curves.pop(i)
                item.hide()
                item.setParent(None)

                item = self.plot.view_boxes.pop(i)
                item.hide()
                item.setParent(None)

                sig = self.plot.signals.pop(i)

                if self.info_index == sig._index:
                    self.info_index = None

            rows = self.channel_selection.count()

            if not rows:
                self.close_request.emit()
            else:
                if self.info_index is None:
                    self.info_index = self.plot.signals[0]._index

                self.plot.set_current_index(self.info_index)
                stats = self.plot.get_stats(self.info_index)
                self.info.set_stats(stats)

        def cursor_move_finished(self):
            x = self.plot.timebase
            if x is not None and len(x):
                dim = len(x)
                position = self.plot.cursor1.value()

                right = np.searchsorted(x, position, side="right")
                if right == 0:
                    next_pos = x[0]
                elif right == dim:
                    next_pos = x[-1]
                else:
                    if position - x[right - 1] < x[right] - position:
                        next_pos = x[right - 1]
                    else:
                        next_pos = x[right]
                self.plot.cursor1.setPos(next_pos)

            # self.plot.cursor_hint.setData(x=[], y=[])
            self.plot.cursor_hint.hide()

        def cursor_moved(self):
            position = self.plot.cursor1.value()

            x = self.plot.timebase

            if x is not None and len(x):
                dim = len(x)
                position = self.plot.cursor1.value()

                right = np.searchsorted(x, position, side="right")
                if right == 0:
                    next_pos = x[0]
                elif right == dim:
                    next_pos = x[-1]
                else:
                    if position - x[right - 1] < x[right] - position:
                        next_pos = x[right - 1]
                    else:
                        next_pos = x[right]

                y = []

                _, (hint_min, hint_max) = self.plot.viewbox.viewRange()

                for viewbox, sig, curve in zip(
                    self.plot.view_boxes, self.plot.signals, self.plot.curves
                ):
                    if curve.isVisible():
                        index = np.argwhere(sig.timestamps == next_pos).flatten()
                        if len(index):
                            _, (y_min, y_max) = viewbox.viewRange()

                            sample = sig.samples[index[0]]
                            sample = (sample - y_min) / (y_max - y_min) * (
                                hint_max - hint_min
                            ) + hint_min

                            y.append(sample)

                # self.plot.viewbox.setYRange(hint_min, hint_max, padding=0)
                # self.plot.cursor_hint.setData(x=[next_pos] * len(y), y=y)
                # if not self.plot.cursor_hint.isVisible():
                #     self.plot.cursor_hint.show()

            if not self.plot.region:
                self.cursor_info.setText(f"t = {position:.6f}s")
                for i, signal in enumerate(self.plot.signals):
                    cut_sig = signal.cut(position, position)
                    if signal.plot_texts is None or len(cut_sig) == 0:
                        samples = cut_sig.samples
                        if signal.conversion and hasattr(signal.conversion, "text_0"):
                            samples = signal.conversion.convert(samples)
                            if samples.dtype.kind == 'S':
                                try:
                                    samples = [s.decode("utf-8") for s in samples]
                                except:
                                    samples = [s.decode("latin-1") for s in samples]
                            else:
                                samples = samples.tolist()
                    else:
                        t = np.argwhere(signal.plot_timestamps == cut_sig.timestamps).flatten()
                        try:
                            samples = [e.decode("utf-8") for e in signal.plot_texts[t]]
                        except:
                            samples = [e.decode("latin-1") for e in signal.plot_texts[t]]

                    item = self.channel_selection.item(i)
                    item = self.channel_selection.itemWidget(item)

                    item.setPrefix("= ")
                    item.setFmt(signal.format)

                    if len(samples):
                        item.setValue(samples[0])
                    else:
                        item.setValue("n.a.")

            if self.info.isVisible():
                stats = self.plot.get_stats(self.info_index)
                self.info.set_stats(stats)

        def cursor_removed(self):
            for i, signal in enumerate(self.plot.signals):
                item = self.channel_selection.item(i)
                item = self.channel_selection.itemWidget(item)

                if not self.plot.region:
                    self.cursor_info.setText("")
                    item.setPrefix("")
                    item.setValue("")
            if self.info.isVisible():
                stats = self.plot.get_stats(self.info_index)
                self.info.set_stats(stats)

        def range_modified(self):
            start, stop = self.plot.region.getRegion()

            self.cursor_info.setText(
                (
                    "< html > < head / > < body >"
                    f"< p >t1 = {start:.6f}s< / p > "
                    f"< p >t2 = {stop:.6f}s< / p > "
                    f"< p >Δt = {stop - start:.6f}s< / p > "
                    "< / body > < / html >"
                )
            )

            for i, signal in enumerate(self.plot.signals):
                samples = signal.cut(start, stop).samples
                item = self.channel_selection.item(i)
                item = self.channel_selection.itemWidget(item)

                item.setPrefix("Δ = ")
                item.setFmt(signal.format)

                if len(samples):
                    if samples.dtype.kind in "ui":
                        delta = np.int64(np.float64(samples[-1]) - np.float64(samples[0]))
                    else:
                        delta = samples[-1] - samples[0]

                    item.setValue(delta)

                else:
                    item.setValue("n.a.")

            if self.info.isVisible():
                stats = self.plot.get_stats(self.info_index)
                self.info.set_stats(stats)

        def xrange_changed(self):
            if self.info.isVisible():
                stats = self.plot.get_stats(self.info_index)
                self.info.set_stats(stats)

        def range_modified_finished(self):
            start, stop = self.plot.region.getRegion()

            if self.plot.timebase is not None and len(self.plot.timebase):
                timebase = self.plot.timebase
                dim = len(timebase)

                right = np.searchsorted(timebase, start, side="right")
                if right == 0:
                    next_pos = timebase[0]
                elif right == dim:
                    next_pos = timebase[-1]
                else:
                    if start - timebase[right - 1] < timebase[right] - start:
                        next_pos = timebase[right - 1]
                    else:
                        next_pos = timebase[right]
                start = next_pos

                right = np.searchsorted(timebase, stop, side="right")
                if right == 0:
                    next_pos = timebase[0]
                elif right == dim:
                    next_pos = timebase[-1]
                else:
                    if stop - timebase[right - 1] < timebase[right] - stop:
                        next_pos = timebase[right - 1]
                    else:
                        next_pos = timebase[right]
                stop = next_pos

                self.plot.region.setRegion((start, stop))

        def keyPressEvent(self, event):
            if event.key() == QtCore.Qt.Key_M and event.modifiers() == QtCore.Qt.NoModifier:

                ch_size, plt_size, info_size = self.splitter.sizes()

                if self.info.isVisible():
                    self.info.hide()
                    self.splitter.setSizes(
                        (
                            ch_size,
                            plt_size + info_size,
                            0,
                        )
                    )

                else:

                    self.info.show()
                    self.splitter.setSizes(
                        (
                            ch_size,
                            int(0.8 * (plt_size + info_size)),
                            int(0.2 * (plt_size + info_size)),
                        )
                    )

            else:
                self.plot.keyPressEvent(event)

        def range_removed(self):
            for i, signal in enumerate(self.plot.signals):
                item = self.channel_selection.item(i)
                item = self.channel_selection.itemWidget(item)

                item.setPrefix("")
                item.setValue("")
                self.cursor_info.setText("")

            if self.plot.cursor1:
                self.plot.cursor_moved.emit()
            if self.info.isVisible():
                stats = self.plot.get_stats(self.info_index)
                self.info.set_stats(stats)

        def computation_channel_inserted(self):
            sig = self.plot.signals[-1]
            sig._index = self._available_index

            if sig.empty:
                name, unit = sig.name, sig.unit
            else:
                name, unit = sig.name, sig.unit
            item = QtWidgets.QListWidgetItem(self.channel_selection)
            it = ChannelDisplay(self._available_index, unit, sig.samples.dtype.kind, 3, self)
            it.setAttribute(QtCore.Qt.WA_StyledBackground)

            self._available_index += 1

            it.setName(name)
            it.setValue("")
            it.setColor(sig.color)
            item.setSizeHint(it.sizeHint())
            self.channel_selection.addItem(item)
            self.channel_selection.setItemWidget(item, it)

            it.color_changed.connect(self.plot.setColor)
            it.enable_changed.connect(self.plot.setSignalEnable)
            it.ylink_changed.connect(self.plot.setCommonAxis)

            it.enable_changed.emit(sig._index, 1)
            it.enable_changed.emit(sig._index, 0)
            it.enable_changed.emit(sig._index, 1)

            self.info_index = sig._index

            self.plot.set_current_index(self.info_index, True)

        def add_new_channels(self, channels):

            self.plot.add_new_channels(channels)

            for sig in channels:
                sig._index = self._available_index

                item = QtWidgets.QListWidgetItem(self.channel_selection)
                item.setData(QtCore.Qt.UserRole, sig.name)
                it = ChannelDisplay(self._available_index, sig.unit, sig.samples.dtype.kind, 3, self)
                it.setAttribute(QtCore.Qt.WA_StyledBackground)

                self._available_index += 1

                it.setName(sig.name)
                it.setValue("")
                it.setColor(sig.color)
                item.setSizeHint(it.sizeHint())
                self.channel_selection.addItem(item)
                self.channel_selection.setItemWidget(item, it)

                it.color_changed.connect(self.plot.setColor)
                it.enable_changed.connect(self.plot.setSignalEnable)
                it.ylink_changed.connect(self.plot.setCommonAxis)

                it.enable_changed.emit(sig._index, 1)
                it.enable_changed.emit(sig._index, 0)
                it.enable_changed.emit(sig._index, 1)

                self.info_index = sig._index

            if channels:
                self.plot.set_current_index(self.info_index)

        def to_config(self):
            count = self.channel_selection.count()

            channels = []
            for i in range(count):
                channel = {}
                item = self.channel_selection.itemWidget(self.channel_selection.item(i))
                name = item._name

                sig = [
                    s
                    for s in self.plot.signals
                    if s.name == name
                ][0]

                channel['name'] = name
                channel['unit'] = sig.unit
                channel['enabled'] = item.display.checkState() == QtCore.Qt.Checked
                channel['common_axis'] = item.ylink.checkState() == QtCore.Qt.Checked
                channel['color'] = sig.color
                channel['computed'] = sig.computed
                ranges = [
                    {
                        'start': start,
                        'stop': stop,
                        'color': color,
                    }
                    for (start, stop), color in item.ranges.items()
                ]
                channel['ranges'] = ranges

                channel['precision'] = item.precision
                channel['fmt'] = item.fmt
                if sig.computed:
                    channel['computation'] = sig.computation

                channels.append(channel)

            config = {
                'channels': channels,
            }

            return config


    class _Plot(pg.PlotWidget):
        cursor_moved = QtCore.pyqtSignal()
        cursor_removed = QtCore.pyqtSignal()
        range_removed = QtCore.pyqtSignal()
        range_modified = QtCore.pyqtSignal()
        range_modified_finished = QtCore.pyqtSignal()
        cursor_move_finished = QtCore.pyqtSignal()
        xrange_changed = QtCore.pyqtSignal()
        computation_channel_inserted = QtCore.pyqtSignal()
        curve_clicked = QtCore.pyqtSignal(int)

        def __init__(self, signals=None, with_dots=False, *args, **kwargs):
            super().__init__()
            self.setContentsMargins(0, 0, 0, 0)
            self.xrange_changed.connect(self.xrange_changed_handle)
            self.with_dots = with_dots
            if self.with_dots:
                self.curvetype = pg.PlotDataItem
            else:
                self.curvetype = pg.PlotCurveItem
            self.info = None
            self.current_index = 0

            self.standalone = kwargs.get("standalone", False)

            self.region = None
            self.cursor1 = None
            self.cursor2 = None
            self.signals = signals or []

            self.disabled_keys = set()

            if self.signals:
                self.all_timebase = self.timebase = reduce(
                    np.union1d, (sig.timestamps for sig in self.signals)
                )
            else:
                self.all_timebase = self.timebase = []

            self.showGrid(x=True, y=True)

            self.plot_item = self.plotItem
            self.plot_item.hideAxis("left")
            self.layout = self.plot_item.layout
            self.scene_ = self.plot_item.scene()
            self.scene_.sigMouseClicked.connect(self._clicked)
            self.viewbox = self.plot_item.vb
            self.viewbox.sigXRangeChanged.connect(self.xrange_changed.emit)

            self.common_axis_items = set()
            self.common_axis_label = ""
            self.common_viewbox = pg.ViewBox(enableMenu=True)
            self.scene_.addItem(self.common_viewbox)
            self.common_viewbox.setXLink(self.viewbox)

            axis = self.layout.itemAt(2, 0)
            axis.setParent(None)
            self.axis = FormatedAxis("left")
            self.layout.removeItem(axis)
            self.layout.addItem(self.axis, 2, 0)
            self.axis.linkToView(axis.linkedView())
            self.plot_item.axes["left"]["item"] = self.axis

            self.cursor_hint = pg.PlotDataItem(
                [],
                [],
                pen="#000000",
                symbolBrush="#000000",
                symbolPen="w",
                symbol="s",
                symbolSize=8,
            )
            self.viewbox.addItem(self.cursor_hint)

            self.view_boxes = []
            self.curves = []

            self.viewbox.sigResized.connect(self.update_views)

            self.keyPressEvent(QtGui.QKeyEvent(QtCore.QEvent.KeyPress, QtCore.Qt.Key_H, QtCore.Qt.NoModifier))

            self.resizeEvent = self._resizeEvent

        def update_lines(self, with_dots=None, force=False):
            with_dots_changed = False

            if with_dots is not None and with_dots != self.with_dots:
                self.with_dots = with_dots
                self.curvetype = pg.PlotDataItem if with_dots else pg.PlotCurveItem
                with_dots_changed = True

            if with_dots_changed or force:
                for i, sig in enumerate(self.signals):
                    color = sig.color
                    t = sig.plot_timestamps

                    if not force:
                        try:
                            curve = self.curvetype(
                                t,
                                sig.plot_samples,
                                pen=color,
                                symbolBrush=color,
                                symbolPen=color,
                                symbol="o",
                                symbolSize=4,
                                clickable=True,
                                mouseWidth=30,
                            )

                            curve.sigClicked.connect(partial(self.curve_clicked.emit, i))
                        except:
                            message = (
                                "Can't show dots due to old pyqtgraph package: "
                                "Please install the latest pyqtgraph from the "
                                "github develop branch\n"
                                "pip install -I --no-deps "
                                "https://github.com/pyqtgraph/pyqtgraph/archive/develop.zip"
                            )
                            logger.warning(message)
                        self.view_boxes[i].removeItem(self.curves[i])

                        self.curves[i] = curve

                        self.view_boxes[i].addItem(curve)
                    else:
                        curve = self.curves[i]
                        curve.setData(x=t, y=sig.plot_samples)

                    if sig.enable:
                        curve.show()
                    else:
                        curve.hide()

        def setColor(self, index, color):
            _, index = self.signal_by_index(index)
            self.signals[index].color = color
            self.curves[index].setPen(color)
            if self.curvetype == pg.PlotDataItem:
                self.curves[index].setSymbolPen(color)
                self.curves[index].setSymbolBrush(color)

            if index == self.current_index:
                self.axis.setPen(color)

        def setCommonAxis(self, index, state):
            viewbox = self.view_boxes[index]
            if state in (QtCore.Qt.Checked, True, 1):
                viewbox.setYRange(*self.common_viewbox.viewRange()[1], padding=0)
                viewbox.setYLink(self.common_viewbox)
                self.common_axis_items.add(index)
            else:
                self.view_boxes[index].setYLink(None)
                self.common_axis_items.remove(index)

            self.common_axis_label = ', '.join(
                self.signals[i].name
                for i in sorted(self.common_axis_items)
            )

            self.set_current_index(self.current_index, True)

        def setSignalEnable(self, index, state):

            _, index = self.signal_by_index(index)

            if state in (QtCore.Qt.Checked, True, 1):
                self.signals[index].enable = True
                self.curves[index].show()
                self.view_boxes[index].setXLink(self.viewbox)
            else:
                self.signals[index].enable = False
                self.curves[index].hide()
                self.view_boxes[index].setXLink(None)

            if self.cursor1:
                self.cursor_move_finished.emit()

        def update_views(self):
            geometry = self.viewbox.sceneBoundingRect()
            for view_box in self.view_boxes:
                view_box.setGeometry(geometry)

        def get_stats(self, index):
            stats = {}
            sig, index = self.signal_by_index(index)
            x = sig.timestamps
            size = len(x)

            if size:

                if sig.plot_texts is not None:
                    stats["overall_min"] = ""
                    stats["overall_max"] = ""
                    stats["overall_average"] = ""
                    stats["overall_rms"] = ""
                    stats["overall_start"] = sig.timestamps[0]
                    stats["overall_stop"] = sig.timestamps[-1]
                    stats["unit"] = ""
                    stats["color"] = sig.color
                    stats["name"] = sig.name

                    if self.cursor1:
                        position = self.cursor1.value()
                        stats["cursor_t"] = position

                        if x[0] <= position <= x[-1]:
                            idx = np.searchsorted(x, position)
                            text = sig.texts[idx]
                            try:
                                text = text.decode("utf-8")
                            except:
                                text = text.decode("latin-1")
                            stats["cursor_value"] = text

                        else:
                            stats["cursor_value"] = "n.a."

                    else:
                        stats["cursor_t"] = ""
                        stats["cursor_value"] = ""

                    stats["selected_start"] = ""
                    stats["selected_stop"] = ""
                    stats["selected_delta_t"] = ""
                    stats["selected_min"] = ""
                    stats["selected_max"] = ""
                    stats["selected_average"] = ""
                    stats["selected_rms"] = ""
                    stats["selected_delta"] = ""
                    stats["visible_min"] = ""
                    stats["visible_max"] = ""
                    stats["visible_average"] = ""
                    stats["visible_rms"] = ""
                    stats["visible_delta"] = ""
                else:
                    stats["overall_min"] = sig.min
                    stats["overall_max"] = sig.max
                    stats["overall_average"] = sig.avg
                    stats["overall_rms"] = sig.rms
                    stats["overall_start"] = sig.timestamps[0]
                    stats["overall_stop"] = sig.timestamps[-1]
                    stats["unit"] = sig.unit
                    stats["color"] = sig.color
                    stats["name"] = sig.name

                    if self.cursor1:
                        position = self.cursor1.value()
                        stats["cursor_t"] = position

                        if x[0] <= position <= x[-1]:
                            idx = np.searchsorted(x, position)
                            val = sig.samples[idx]
                            if sig.conversion and hasattr(sig.conversion,"text_0"):
                                vals = np.array([val])
                                vals = sig.conversion.convert(vals)
                                if vals.dtype.kind == 'S':
                                    try:
                                        vals = [s.decode("utf-8") for s in vals]
                                    except UnicodeDecodeError:
                                        vals = [s.decode("latin-1") for s in vals]
                                    val = f"{val:.6f}= {vals[0]}"
                                else:
                                    val = f"{val:.6f}= {vals[0]:.6f}"

                            stats["cursor_value"] = val

                        else:
                            stats["cursor_value"] = "n.a."

                    else:
                        stats["cursor_t"] = ""
                        stats["cursor_value"] = ""

                    if self.region:
                        start, stop = self.region.getRegion()

                        if sig._stats["range"] != (start, stop):
                            new_stats = {}
                            new_stats["selected_start"] = start
                            new_stats["selected_stop"] = stop
                            new_stats["selected_delta_t"] = stop - start

                            cut = sig.cut(start, stop)

                            if len(cut):
                                new_stats["selected_min"] = np.nanmin(cut.samples)
                                new_stats["selected_max"] = np.nanmax(cut.samples)
                                new_stats["selected_average"] = np.mean(cut.samples)
                                new_stats["selected_rms"] = (
                                     np.sqrt(np.mean(np.square(cut.samples)))
                                )
                                if cut.samples.dtype.kind in "ui":
                                    new_stats["selected_delta"] = int(
                                        float(cut.samples[-1]) - (cut.samples[0])
                                    )
                                else:
                                    new_stats["selected_delta"] = (
                                        cut.samples[-1] - cut.samples[0]
                                    )

                            else:
                                new_stats["selected_min"] = "n.a."
                                new_stats["selected_max"] = "n.a."
                                new_stats["selected_average"] = "n.a."
                                new_stats["selected_rms"] = "n.a."
                                new_stats["selected_delta"] = "n.a."

                            sig._stats["range"] = (start, stop)
                            sig._stats["range_stats"] = new_stats

                        stats.update(sig._stats["range_stats"])

                    else:
                        stats["selected_start"] = ""
                        stats["selected_stop"] = ""
                        stats["selected_delta_t"] = ""
                        stats["selected_min"] = ""
                        stats["selected_max"] = ""
                        stats["selected_average"] = ""
                        stats["selected_rms"] = ""
                        stats["selected_delta"] = ""

                    (start, stop), _ = self.viewbox.viewRange()

                    if sig._stats["visible"] != (start, stop):
                        new_stats = {}
                        new_stats["visible_start"] = start
                        new_stats["visible_stop"] = stop
                        new_stats["visible_delta_t"] = stop - start

                        cut = sig.cut(start, stop)

                        if len(cut):
                            new_stats["visible_min"] = np.nanmin(cut.samples)
                            new_stats["visible_max"] = np.nanmax(cut.samples)
                            new_stats["visible_average"] = np.mean(cut.samples)
                            new_stats["visible_rms"] = (
                                 np.sqrt(np.mean(np.square(cut.samples)))
                            )
                            if cut.samples.dtype.kind in 'ui':
                                new_stats["visible_delta"] = (
                                    int(cut.samples[-1]) - int(cut.samples[0])
                                )
                            else:
                                new_stats["visible_delta"] = (
                                    cut.samples[-1] - cut.samples[0]
                                )

                        else:
                            new_stats["visible_min"] = "n.a."
                            new_stats["visible_max"] = "n.a."
                            new_stats["visible_average"] = "n.a."
                            new_stats["visible_rms"] = "n.a."
                            new_stats["visible_delta"] = "n.a."

                        sig._stats["visible"] = (start, stop)
                        sig._stats["visible_stats"] = new_stats

                    stats.update(sig._stats["visible_stats"])

            else:
                stats["overall_min"] = "n.a."
                stats["overall_max"] = "n.a."
                stats["overall_average"] = "n.a."
                stats["overall_rms"] = "n.a."
                stats["overall_start"] = "n.a."
                stats["overall_stop"] = "n.a."
                stats["unit"] = sig.unit
                stats["color"] = sig.color
                stats["name"] = sig.name

                if self.cursor1:
                    position = self.cursor1.value()
                    stats["cursor_t"] = position

                    stats["cursor_value"] = "n.a."

                else:
                    stats["cursor_t"] = ""
                    stats["cursor_value"] = ""

                if self.region:
                    start, stop = self.region.getRegion()

                    stats["selected_start"] = start
                    stats["selected_stop"] = stop
                    stats["selected_delta_t"] = stop - start

                    stats["selected_min"] = "n.a."
                    stats["selected_max"] = "n.a."
                    stats["selected_average"] = "n.a."
                    stats["selected_rms"] = "n.a."
                    stats["selected_delta"] = "n.a."

                else:
                    stats["selected_start"] = ""
                    stats["selected_stop"] = ""
                    stats["selected_delta_t"] = ""
                    stats["selected_min"] = ""
                    stats["selected_max"] = ""
                    stats["selected_average"] = "n.a."
                    stats["selected_rms"] = "n.a."
                    stats["selected_delta"] = ""

                (start, stop), _ = self.viewbox.viewRange()

                stats["visible_start"] = start
                stats["visible_stop"] = stop
                stats["visible_delta_t"] = stop - start

                stats["visible_min"] = "n.a."
                stats["visible_max"] = "n.a."
                stats["visible_average"] = "n.a."
                stats["visible_rms"] = "n.a."
                stats["visible_delta"] = "n.a."

            return stats

        def keyPressEvent(self, event):
            key = event.key()
            modifier = event.modifiers()

            if key in self.disabled_keys:
                super().keyPressEvent(event)
            else:

                if key == QtCore.Qt.Key_C:
                    if self.cursor1 is None:
                        start, stop = self.viewbox.viewRange()[0]
                        self.cursor1 = Cursor(pos=0, angle=90, movable=True)
                        self.plotItem.addItem(self.cursor1, ignoreBounds=True)
                        self.cursor1.sigPositionChanged.connect(self.cursor_moved.emit)
                        self.cursor1.sigPositionChangeFinished.connect(
                            self.cursor_move_finished.emit
                        )
                        self.cursor1.setPos((start + stop) / 2)
                        self.cursor_move_finished.emit()

                    else:
                        self.plotItem.removeItem(self.cursor1)
                        self.cursor1.setParent(None)
                        self.cursor1 = None
                        self.cursor_removed.emit()

                elif key == QtCore.Qt.Key_F:
                    for i, (viewbox, signal) in enumerate(zip(self.view_boxes, self.signals)):
                        if len(signal.plot_samples):
                            min_, max_ = (
                                np.nanmin(signal.plot_samples),
                                np.nanmax(signal.plot_samples),
                            )
                            if min_ == -float('inf') and max_ == float('inf'):
                                min_ = 0
                                max_ = 1
                            elif min_ == -float('inf'):
                                min_ = max_ - 1
                            elif max_ == float('inf'):
                                max_ = min_ + 1
                            viewbox.setYRange(min_, max_, padding=0)

                    if self.cursor1:
                        self.cursor_moved.emit()

                elif key == QtCore.Qt.Key_G:
                    if self.plotItem.ctrl.yGridCheck.isChecked():
                        self.showGrid(x=True, y=False)
                    else:
                        self.showGrid(x=True, y=True)

                elif key in (QtCore.Qt.Key_I, QtCore.Qt.Key_O):
                    x_range, _ = self.viewbox.viewRange()
                    delta = x_range[1] - x_range[0]
                    step = delta * 0.05
                    if key == QtCore.Qt.Key_I:
                        step = -step
                    if self.cursor1:
                        pos = self.cursor1.value()
                        x_range = pos - delta / 2, pos + delta / 2
                    self.viewbox.setXRange(
                        x_range[0] - step, x_range[1] + step, padding=0
                    )

                elif key == QtCore.Qt.Key_R:
                    if self.region is None:

                        self.region = pg.LinearRegionItem((0, 0))
                        self.region.setZValue(-10)
                        self.plotItem.addItem(self.region)
                        self.region.sigRegionChanged.connect(self.range_modified.emit)
                        self.region.sigRegionChangeFinished.connect(
                            self.range_modified_finished.emit
                        )
                        for line in self.region.lines:
                            line.addMarker("^", 0)
                            line.addMarker("v", 1)
                        start, stop = self.viewbox.viewRange()[0]
                        start, stop = (
                            start + 0.1 * (stop - start),
                            stop - 0.1 * (stop - start),
                        )
                        self.region.setRegion((start, stop))

                    else:
                        self.region.setParent(None)
                        self.region.hide()
                        self.region = None
                        self.range_removed.emit()

                elif key == QtCore.Qt.Key_S and modifier == QtCore.Qt.ControlModifier:
                    file_name, _ = QtWidgets.QFileDialog.getSaveFileName(
                        self,
                        "Select output measurement file", "",
                        "MDF version 4 files (*.mf4)",
                    )

                    if file_name:
                        with MDF() as mdf:
                            mdf.append(self.signals)
                            mdf.save(file_name, overwrite=True)

                elif key == QtCore.Qt.Key_S and modifier == QtCore.Qt.NoModifier:
                    count = len(
                        [
                            sig
                            for (sig, curve) in zip(self.signals, self.curves)
                            if not sig.empty and curve.isVisible()
                        ]
                    )

                    if count:
                        position = 0
                        for signal, viewbox, curve in zip(
                            reversed(self.signals),
                            reversed(self.view_boxes),
                            reversed(self.curves),
                        ):
                            if not signal.empty and curve.isVisible():
                                min_ = signal.min
                                max_ = signal.max
                                if min_ == max_:
                                    min_, max_ = min_ - 1, max_ + 1

                                dim = (float(max_) - min_) * 1.1

                                max_ = min_ + dim * count
                                min_, max_ = (
                                    min_ - dim * position,
                                    max_ - dim * position,
                                )

                                viewbox.setYRange(min_, max_, padding=0)

                                position += 1
                    else:
                        xrange, _ = self.viewbox.viewRange()
                        self.viewbox.autoRange(padding=0)
                        self.viewbox.setXRange(*xrange, padding=0)
                        self.viewbox.disableAutoRange()
                    if self.cursor1:
                        self.cursor_moved.emit()

                elif key == QtCore.Qt.Key_H and modifier == QtCore.Qt.ControlModifier:
                    for i, signal in enumerate(self.signals):
                        if signal.samples.dtype.kind in "ui":
                            signal.format = "hex"
                        if self.current_index == i:
                            self.axis.format = "hex"
                            self.axis.hide()
                            self.axis.show()
                    if self.cursor1:
                        self.cursor_moved.emit()

                elif key == QtCore.Qt.Key_B and modifier == QtCore.Qt.ControlModifier:
                    for i, signal in enumerate(self.signals):
                        if signal.samples.dtype.kind in "ui":
                            signal.format = "bin"
                        if self.current_index == i:
                            self.axis.format = "bin"
                            self.axis.hide()
                            self.axis.show()
                    if self.cursor1:
                        self.cursor_moved.emit()

                elif key == QtCore.Qt.Key_P and modifier == QtCore.Qt.ControlModifier:
                    for i, signal in enumerate(self.signals):
                        if signal.samples.dtype.kind in "ui":
                            signal.format = "phys"
                        if self.current_index == i:
                            self.axis.format = "phys"
                            self.axis.hide()
                            self.axis.show()
                    if self.cursor1:
                        self.cursor_moved.emit()

                elif key in (QtCore.Qt.Key_Left, QtCore.Qt.Key_Right):
                    if self.cursor1:
                        prev_pos = pos = self.cursor1.value()
                        dim = len(self.timebase)
                        if dim:
                            pos = np.searchsorted(self.timebase, pos)
                            if key == QtCore.Qt.Key_Right:
                                pos += 1
                            else:
                                pos -= 1
                            pos = np.clip(pos, 0, dim - 1)
                            pos = self.timebase[pos]
                        else:
                            if key == QtCore.Qt.Key_Right:
                                pos += 1
                            else:
                                pos -= 1

                        (left_side, right_side), _ = self.viewbox.viewRange()

                        if pos >= right_side:
                            delta = abs(pos - prev_pos)
                            self.viewbox.setXRange(
                                left_side + delta, right_side + delta, padding=0
                            )
                        elif pos <= left_side:
                            delta = abs(pos - prev_pos)
                            self.viewbox.setXRange(
                                left_side - delta, right_side - delta, padding=0
                            )
                        else:
                            delta = 0

                        self.cursor1.setValue(pos)

                elif key == QtCore.Qt.Key_H:
                    start_ts = [
                        sig.timestamps[0]
                        for sig in self.signals
                        if len(sig.timestamps)
                    ]

                    stop_ts = [
                        sig.timestamps[-1]
                        for sig in self.signals
                        if len(sig.timestamps)
                    ]

                    if start_ts:
                        start_t, stop_t = min(start_ts), max(stop_ts)

                        self.viewbox.setXRange(start_t, stop_t)
                        self.viewbox.autoRange(padding=0)
                        self.viewbox.disableAutoRange()
                        if self.cursor1:
                            self.cursor_moved.emit()

                elif key == QtCore.Qt.Key_Insert:
                    dlg = DefineChannel(self.signals, self.all_timebase, self)
                    dlg.setModal(True)
                    dlg.exec_()
                    sig = dlg.result

                    if sig is not None:
                        self.add_new_channels([sig], computed=True)
                        self.computation_channel_inserted.emit()

                else:
                    super().keyPressEvent(event)

        def trim(self, width, start, stop, signals):
            for sig in signals:
                dim = len(sig.samples)
                if dim:

                    start_t, stop_t = (
                        sig.timestamps[0],
                        sig.timestamps[-1],
                    )
                    if start > stop_t or stop < start_t:
                        sig.plot_samples = sig.samples[:0]
                        sig.plot_timestamps = sig.timestamps[:0]
                        if sig.plot_texts is not None:
                            sig.plot_texts = sig.texts[:0]
                    else:
                        start_ = max(start, start_t)
                        stop_ = min(stop, stop_t)

                        visible = abs(int((stop_ - start_) / (stop - start) * width))

                        start_ = np.searchsorted(
                            sig.timestamps, start_, side="right"
                        )
                        stop_ = np.searchsorted(
                            sig.timestamps, stop_, side="right"
                        )

                        if visible:
                            raster = abs((stop_ - start_)) // visible
                        else:
                            raster = 0
                        if raster:
                            rows = (stop_ - start_) // raster
                            stop_2 = start_ + rows * raster

                            samples = sig.samples[start_: stop_2].reshape(rows, raster)
                            max_ = np.nanmax(samples, axis=1)
                            min_ = np.nanmin(samples, axis=1)
                            samples = np.dstack((min_, max_)).ravel()

                            timestamps = sig.timestamps[start_: stop_2].reshape(rows, raster)
                            t_max = np.nanmax(timestamps, axis=1)
                            t_min = np.nanmin(timestamps, axis=1)

                            timestamps = np.dstack((t_min, t_max)).ravel()

                            if stop_2 != stop_:
                                samples_ = sig.samples[stop_2: stop_]
                                max_ = np.nanmax(samples_)
                                min_ = np.nanmin(samples_)
                                samples = np.concatenate((samples, [min_, max_]))

                                timestamps_ = sig.timestamps[stop_2: stop_]
                                t_max = np.nanmax(timestamps_)
                                t_min = np.nanmin(timestamps_)

                                timestamps = np.concatenate((timestamps, [t_min, t_max]))

                            sig.plot_samples = samples
                            sig.plot_timestamps = timestamps
                            if sig.plot_texts is not None:

                                idx = np.searchsorted(sig.timestamps, timestamps)
                                sig.plot_texts = sig.texts[idx]
                        else:
                            start_ = max(0, start_ - 2)
                            stop_ += 2

                            sig.plot_samples = sig.samples[start_:stop_]
                            sig.plot_timestamps = sig.timestamps[start_:stop_]
                            if sig.plot_texts is not None:
                                sig.plot_texts = sig.texts[start_:stop_]

        def xrange_changed_handle(self):
            (start, stop), _ = self.viewbox.viewRange()

            width = self.width() - self.axis.width()
            self.trim(width, start, stop, self.signals)

            self.update_lines(force=True)

        def _resizeEvent(self, ev):
            self.xrange_changed_handle()
            super().resizeEvent(ev)

        def set_current_index(self, index, force=False):
            axis = self.axis
            viewbox = self.viewbox

            sig, index = self.signal_by_index(index)

            if sig.conversion and hasattr(sig.conversion, "text_0"):
                axis.text_conversion = sig.conversion
            else:
                axis.text_conversion = None
            axis.format = sig.format

            if index in self.common_axis_items:
                if self.current_index not in self.common_axis_items or force:
                    for i, vbox in enumerate(self.view_boxes):
                        if i not in self.common_axis_items:
                            vbox.setYLink(None)

                    vbox = self.view_boxes[index]
                    viewbox.setYRange(*vbox.viewRange()[1], padding=0)
                    self.common_viewbox.setYRange(*vbox.viewRange()[1], padding=0)
                    self.common_viewbox.setYLink(viewbox)

                    axis.setPen("#FFFFFF")
                    axis.setLabel(self.common_axis_label)

            else:
                self.common_viewbox.setYLink(None)
                for i, vbox in enumerate(self.view_boxes):
                    if i not in self.common_axis_items:
                        vbox.setYLink(None)

                viewbox.setYRange(*self.view_boxes[index].viewRange()[1], padding=0)
                self.view_boxes[index].setYLink(viewbox)
                if len(sig.name) <= 32:
                    axis.setLabel(f'{sig.name} [{sig.unit}]')
                else:
                    axis.setLabel(f"{sig.name[:29]}...  [{sig.unit}]")
                axis.setPen(sig.color)
                axis.update()

            self.current_index = index

        def _clicked(self, event):
            if QtCore.Qt.Key_C not in self.disabled_keys:
                x = self.plot_item.vb.mapSceneToView(event.scenePos())

                if self.cursor1 is not None:
                    self.plotItem.removeItem(self.cursor1)
                    self.cursor1.setParent(None)
                    self.cursor1 = None

                self.cursor1 = Cursor(pos=x, angle=90, movable=True)
                self.plotItem.addItem(self.cursor1, ignoreBounds=True)
                self.cursor1.sigPositionChanged.connect(self.cursor_moved.emit)
                self.cursor1.sigPositionChangeFinished.connect(
                    self.cursor_move_finished.emit
                )
                self.cursor_move_finished.emit()

        def add_new_channels(self, channels, computed=False):
            geometry = self.viewbox.sceneBoundingRect()
            for i, sig in enumerate(channels):
                index = len(self.signals)

                self.signals.append(sig)

                if sig.samples.dtype.kind == "f":
                    sig.format = "phys"
                    sig.plot_texts = None
                else:
                    sig.format = "phys"
                    if sig.samples.dtype.kind in "SV":
                        sig.plot_texts = sig.texts = sig.samples
                        sig.samples = np.zeros(len(sig.samples))
                    else:
                        sig.plot_texts = None
                sig.enable = True
                if not hasattr(sig, 'computed'):
                    sig.computed = computed
                    if not computed:
                        sig.computation = {}

                if sig.conversion:
                    vals = sig.conversion.convert(sig.samples)
                    if vals.dtype.kind != 'S':
                        nans = np.isnan(vals)
                        samples = np.where(
                            nans,
                            sig.samples,
                            vals,
                        )
                        sig.samples = samples

                sig.plot_samples = sig.samples
                sig.plot_timestamps = sig.timestamps

                sig._stats = {
                    "range": (0, -1),
                    "range_stats": {},
                    "visible": (0, -1),
                    "visible_stats": {},
                }

                color = COLORS[index % 10]
                sig.color = color

                if len(sig.samples):
                    sig.min = np.nanmin(sig.samples)
                    sig.max = np.nanmax(sig.samples)
                    sig.avg = np.mean(sig.samples)
                    sig.rms = np.sqrt(np.mean(np.square(sig.samples)))
                    sig.empty = False

                    if sig.min == -float('inf') and sig.max == float('inf'):
                        sig.min = 0
                        sig.max = 1
                    elif sig.min == -float('inf'):
                        sig.min = sig.max - 1
                    elif sig.max == float('inf'):
                        sig.max = sig.min + 1

                else:
                    sig.empty = True
                    sig.min = 'n.a.'
                    sig.max = 'n.a.'
                    sig.rms = 'n.a.'
                    sig.avg = 'n.a.'

                view_box = pg.ViewBox(enableMenu=False)
                view_box.disableAutoRange()

                self.scene_.addItem(view_box)

                t = sig.plot_timestamps

                curve = self.curvetype(
                    t,
                    sig.plot_samples,
                    pen=color,
                    symbolBrush=color,
                    symbolPen=color,
                    symbol="o",
                    symbolSize=4,
                    clickable=True,
                    mouseWidth=30,
                )
                curve.sigClicked.connect(partial(self.curve_clicked.emit, index))
                view_box.addItem(curve)

                view_box.setXLink(self.viewbox)
                self.view_boxes.append(view_box)
                self.curves.append(curve)
                curve.show()

                if not sig.empty:
                    view_box.setYRange(sig.min, sig.max, padding=0, update=True)
                (start, stop), _ = self.viewbox.viewRange()
                view_box.setXRange(start, stop, padding=0, update=True)

                view_box.setGeometry(geometry)

            if len(self.all_timebase):
                home = False
            else:
                home = True

            if self.signals:
                self.all_timebase = self.timebase = reduce(
                    np.union1d, (sig.timestamps for sig in self.signals)
                )
            else:
                self.all_timebase = self.timebase = None

            if home:
                self.keyPressEvent(QtGui.QKeyEvent(QtCore.QEvent.KeyPress, QtCore.Qt.Key_H, QtCore.Qt.NoModifier))

            QtWidgets.QApplication.processEvents()

        def signal_by_index(self, index):
            for i, sig in enumerate(self.signals):
                if sig._index == index:
                    return sig, i

            raise Exception('Signal not found')

except ImportError:
    raise
    PYQTGRAPH_AVAILABLE = False



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'd:\DSUsers\uidn3651\02__PythonWorkspace\asammdf\asammdf\gui\ui\channel_info_widget.ui'
#
# Created by: PyQt5 UI code generator 5.12.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_ChannelInfo(object):
    def setupUi(self, ChannelInfo):
        ChannelInfo.setObjectName("ChannelInfo")
        ChannelInfo.resize(926, 558)
        self.horizontalLayout = QtWidgets.QHBoxLayout(ChannelInfo)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.groupBox = QtWidgets.QGroupBox(ChannelInfo)
        self.groupBox.setFlat(False)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.scrollArea = QtWidgets.QScrollArea(self.groupBox)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 277, 505))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.scrollAreaWidgetContents)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.channel_label = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.channel_label.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByKeyboard|QtCore.Qt.TextSelectableByMouse)
        self.channel_label.setObjectName("channel_label")
        self.verticalLayout_3.addWidget(self.channel_label)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacerItem)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.verticalLayout_4.addWidget(self.scrollArea)
        self.horizontalLayout.addWidget(self.groupBox)
        self.groupBox_2 = QtWidgets.QGroupBox(ChannelInfo)
        self.groupBox_2.setObjectName("groupBox_2")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.groupBox_2)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.scrollArea_2 = QtWidgets.QScrollArea(self.groupBox_2)
        self.scrollArea_2.setWidgetResizable(True)
        self.scrollArea_2.setObjectName("scrollArea_2")
        self.scrollAreaWidgetContents_2 = QtWidgets.QWidget()
        self.scrollAreaWidgetContents_2.setGeometry(QtCore.QRect(0, 0, 276, 505))
        self.scrollAreaWidgetContents_2.setObjectName("scrollAreaWidgetContents_2")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.scrollAreaWidgetContents_2)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.conversion_label = QtWidgets.QLabel(self.scrollAreaWidgetContents_2)
        self.conversion_label.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByKeyboard|QtCore.Qt.TextSelectableByMouse)
        self.conversion_label.setObjectName("conversion_label")
        self.verticalLayout_2.addWidget(self.conversion_label)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem1)
        self.scrollArea_2.setWidget(self.scrollAreaWidgetContents_2)
        self.verticalLayout_6.addWidget(self.scrollArea_2)
        self.horizontalLayout.addWidget(self.groupBox_2)
        self.groupBox_3 = QtWidgets.QGroupBox(ChannelInfo)
        self.groupBox_3.setObjectName("groupBox_3")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.groupBox_3)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.scrollArea_3 = QtWidgets.QScrollArea(self.groupBox_3)
        self.scrollArea_3.setWidgetResizable(True)
        self.scrollArea_3.setObjectName("scrollArea_3")
        self.scrollAreaWidgetContents_3 = QtWidgets.QWidget()
        self.scrollAreaWidgetContents_3.setGeometry(QtCore.QRect(0, 0, 277, 505))
        self.scrollAreaWidgetContents_3.setObjectName("scrollAreaWidgetContents_3")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.scrollAreaWidgetContents_3)
        self.verticalLayout.setObjectName("verticalLayout")
        self.source_label = QtWidgets.QLabel(self.scrollAreaWidgetContents_3)
        self.source_label.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByKeyboard|QtCore.Qt.TextSelectableByMouse)
        self.source_label.setObjectName("source_label")
        self.verticalLayout.addWidget(self.source_label)
        spacerItem2 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem2)
        self.scrollArea_3.setWidget(self.scrollAreaWidgetContents_3)
        self.verticalLayout_5.addWidget(self.scrollArea_3)
        self.horizontalLayout.addWidget(self.groupBox_3)

        self.retranslateUi(ChannelInfo)
        QtCore.QMetaObject.connectSlotsByName(ChannelInfo)

    def retranslateUi(self, ChannelInfo):
        _translate = QtCore.QCoreApplication.translate
        ChannelInfo.setWindowTitle(_translate("ChannelInfo", "Form"))
        self.groupBox.setTitle(_translate("ChannelInfo", "Channel"))
        self.channel_label.setText(_translate("ChannelInfo", "Channel metadata"))
        self.groupBox_2.setTitle(_translate("ChannelInfo", "Conversion"))
        self.conversion_label.setText(_translate("ChannelInfo", "No conversion"))
        self.groupBox_3.setTitle(_translate("ChannelInfo", "Source"))
        self.source_label.setText(_translate("ChannelInfo", "No source"))


